// Copyright (c) ONNX Project Contributors
//
// SPDX-License-Identifier: Apache-2.0

#pragma once

namespace ONNX_NAMESPACE {
extern const char kDoc_Acos_ver7[];
extern const char kDoc_Acosh_ver9[];
extern const char kDoc_Asin_ver7[];
extern const char kDoc_Asinh_ver9[];
extern const char kDoc_Atan_ver7[];
extern const char kDoc_Atanh_ver9[];
extern const char kDoc_Bernoulli_ver15[];
extern const char kDoc_BitCast_ver26[];
extern const char kDoc_Cast_ver24[];
extern const char kDoc_CastLike_ver24[];
extern const char kDoc_Compress_ver9[];
extern const char kDoc_Constant_ver24[];
extern const char kDoc_ConstantOfShape_ver24[];
extern const char kDoc_Cos_ver7[];
extern const char kDoc_Cosh_ver9[];
extern const char kDoc_CumSum_ver11[];
extern const char kDoc_DeformConv_ver19[];
extern const char kDoc_DequantizeLinear_ver24[];
extern const char kDoc_Det_ver11[];
extern const char kDoc_Dropout_ver13[];
extern const char kDoc_Elu_ver6[];
extern const char kDoc_Erf_ver9[];
extern const char kDoc_Exp_ver6[];
extern const char kDoc_Expand_ver8[];
extern const char kDoc_EyeLike_ver9[];
extern const char kDoc_Flatten_ver24[];
extern const char kDoc_GridSample_ver20[];
extern const char kDoc_GRU_ver14[];
extern const char kDoc_HardSigmoid_ver6[];
extern const char kDoc_HardSwish_ver14[];
extern const char kDoc_InstanceNormalization_ver6[];
extern const char kDoc_LeakyRelu_ver1[];
extern const char kDoc_Log_ver6[];
extern const char kDoc_Loop_ver23[];
extern const char kDoc_LpNormalization_ver1[];
extern const char kDoc_LSTM_ver14[];
extern const char kDoc_MatMul_ver9[];
extern const char kDoc_MaxUnpool_ver11[];
extern const char kDoc_mish_ver18[];
extern const char kDoc_Multinomial_ver7[];
extern const char kDoc_Neg_ver6[];
extern const char kDoc_NegativeLogLikelihoodLoss_ver13[];
extern const char kDoc_NonMaxSuppression_ver10[];
extern const char kDoc_Pad_ver24[];
extern const char kDoc_Pow_ver13[];
extern const char kDoc_PRelu_ver7[];
extern const char kDoc_RandomNormal_ver1[];
extern const char kDoc_RandomNormalLike_ver1[];
extern const char kDoc_RandomUniform_ver1[];
extern const char kDoc_RandomUniformLike_ver1[];
extern const char kDoc_Reciprocal_ver6[];
extern const char kDoc_Relu_ver6[];
extern const char kDoc_Reshape_ver24[];
extern const char kDoc_RNN_ver14[];
extern const char kDoc_RoiAlign_ver16[];
extern const char kDoc_Round_ver11[];
extern const char kDoc_scan_24[];
extern const char kDoc_Selu_ver6[];
extern const char kDoc_Shape_ver24[];
extern const char kDoc_Sigmoid_ver6[];
extern const char kDoc_Sign_ver9[];
extern const char kDoc_Sin_ver7[];
extern const char kDoc_Sinh_ver9[];
extern const char kDoc_Size_ver24[];
extern const char kDoc_Softplus_ver1[];
extern const char kDoc_Softsign_ver1[];
extern const char kDoc_SpaceToDepth_ver1[];
extern const char kDoc_Sqrt_ver6[];
extern const char kDoc_Squeeze_ver24[];
extern const char kDoc_Tan_ver7[];
extern const char kDoc_Tanh_ver6[];
extern const char kDoc_ThresholdedRelu_ver10[];
extern const char kDoc_Tile_ver6[];
extern const char kDoc_Unsqueeze_ver24[];
extern const char kDoc_Upsample_ver7[];
extern const char kDoc_Where_ver9[];
} // namespace ONNX_NAMESPACE
